import type { NitroApp } from "nitropack";
import { ApolloServer, HeaderMap } from '@apollo/server'
import { startStandaloneServer } from "@apollo/server/standalone";
import Resolvers from '../../public/apollo/resolvers/query'
import fs from 'fs'

function readGraphqlString (filename: string) {
  const path = './public/apollo/typeDefs/' + filename + '.graphql'
  return fs.readFileSync(path, { encoding: 'utf-8' })
}

export default defineNitroPlugin((nitroApp: NitroApp) => {
  const resolvers = Resolvers
  const query = readGraphqlString('query')
  const user = readGraphqlString('user')
  const v2 = readGraphqlString('v2')
  const v1 = readGraphqlString('v1')
  const typeDefs = `
  ${query}
  ${user}
  ${v2}
  ${v1}
  `
  const server = new ApolloServer({
    typeDefs,
    resolvers
  })
  startStandaloneServer(server).then(() => {
    console.log('apollo started')
  })
  nitroApp.router.use('/graphql', defineEventHandler({
    handler (event) {
      console.log('in graphql')
      const headers = new HeaderMap();
      for (const [key, value] of Object.entries(event.headers)) {
        if (value !== undefined) {
          headers.set(key, Array.isArray(value) ? value.join(', ') : value);
        }
      }
      return server.executeHTTPGraphQLRequest({
        httpGraphQLRequest: {
          method: 'get',
          headers,
          search: `query{
            user
          }`,
          body: {
            username: 'wyq'
          }
        },
        context: () => {
          return Promise.resolve({})
        }
      }).then((value) => {
        console.log('success', value)
        return value
      }).catch(err => {
        console.log('err', err)
      })
    }
  }))
})